@@ -95,7 +95,115 @@ def nano_map2dict(mapping, sep="\t", header = None):
 
     return read_mapping
 
+def nano_groupreads(mapping, OUTPUT="Grouped_reads", groupfile=None, keepSubdir=False, changeName=True, changePrefix="", gzip_extension=False, exist_OK = False):
+    """This function is used to group reads, i.e 16S and ITS by the groupfile, or change folder and read names if necessary
 
+    Args:
+        mapping ([type]): [description]
+        OUTPUT (str, optional): [description]. Defaults to "grouped_reads".
+        groupfile ([type], optional): [provide a tab file to specify groups, need "GROUP" column and "SampleID" should match to the mapping input keys]. Defaults to None
+        keepSubdir (Boolean): if keep the orginial subfolders, or put sub files together by group
+        changeName: change folder name and file name
+        changePrefix: add prefix to folder name and file name
+    """
+    # move files first -> then classify to groups
+
+    os.makedirs(OUTPUT, exist_ok=exist_OK)
+    root_dir = os.path.abspath(OUTPUT)
+
+
+    # add summary report
+    raw_read = defaultdict(list)
+
+    moved_reads = nano_copysource(mapping, OUTPUT=OUTPUT, exist_ok=True)
+
+    # moved_reads : {"path/barcode01":[...]}
+
+    groupGuide = pd.read_csv(groupfile, sep="\t", header=0, names=["SampleID", "Barcodes", "Group"])
+    # get unique groups
+    # {16S: [barcode01, barcode02, ...]}
+    GRPs = defaultdict(list)
+    for row, content in groupGuide.iterrows():
+        GRPs[content["Group"]].append(str(content["Barcodes"])+"_"+str(content["SampleID"]))
+
+    
+    # make subfolders by group
+    for k, v in GRPs.items():
+        # k: group i.e. 16S or ITS
+        # v: barcode_sampleID
+        if not os.path.exists(k):
+            # make group subfoler
+            sam = os.path.join(root_dir, os.path.basename(k))
+            os.makedirs(sam, exist_ok=exist_OK)
+            
+        # extract barcode and sampleID
+        for j in v:
+            
+            b_s = j.split("_")
+            bar = b_s[0]
+            sampleid = b_s[1]
+            
+            # match with moved_reads
+            path_moved = os.path.join(root_dir, bar)
+            reads_need_moving = moved_reads[path_moved] # a list of files with absolute path
+            
+            for mread in reads_need_moving:
+            
+                # if keepsubdir
+                print("\n......keeping sub folders ......\n")
+                if keepSubdir:
+                    # make sub folder under each group folder
+                    if changeName: # change subfolder name
+                        print("\n...rename sub folders and files...\n")
+                        # sam=sample path
+                        new_name =k+"_"+changePrefix+sampleid+"_"+bar
+                        sub_sam = os.path.join(sam, new_name)
+                        if gzip_extension:
+                            chgn_read = os.path.join(sub_sam, new_name+"."+os.path.basename(mread).split(".")[1]+".gz")
+                        else:
+                            chgn_read = os.path.join(sub_sam, new_name+"."+os.path.basename(mread).split(".")[1])
+                    else:
+                        sub_sam = os.path.join(sam, bar)
+                        chgn_read = os.path.join(sub_sam, os.path.basename(mread))
+
+                    if not os.path.exists(sub_sam): # create sub folders
+                        os.makedirs(sub_sam, exist_ok=exist_OK)
+                        
+                        
+                    # now move the reads
+                    raw_read[sub_sam].append(chgn_read)
+                    print("\n.Grouping.\n")
+                    command = " ".join(["cp", mread, chgn_read])
+                    subprocess.run(command, shell=True)
+                    
+                    
+
+                else: # not keepsubdir
+                    print("\n......Not keeping sub folders ......\n")
+                    if changeName: # change subfolder name
+                        new_name =k+"_"+changePrefix+sampleid+"_"+bar
+                        if gzip_extension:
+                            chgn_read = os.path.join(sam, new_name+"."+os.path.basename(mread).split(".")[1]+".gz")
+                        else:
+                            chgn_read = os.path.join(sam, new_name+"."+os.path.basename(mread).split(".")[1])
+                    else:
+                        chgn_read = os.path.join(sam, os.path.basename(mread))
+                                        # now move the reads
+                            
+                    raw_read[sam].append(chgn_read)
+                    print("\n.Grouping.\n")
+                    command = " ".join(["cp", mread, chgn_read])
+                    subprocess.run(command, shell=True)
+
+
+    # remove the copied file to save space
+    print("!!remove copied reads to save space\n")
+    for i in moved_reads.keys():
+        command = " ".join(["rm", i, "-rf"])
+        subprocess.run(command, shell=True)
+
+
+    return raw_read
 
 def nano_copysource(mapping, OUTPUT="copied_fastq_reads", exist_ok=False):
     """Copy source fastq files to current working direction, to avoid mess up original files
@@ -222,11 +330,12 @@ def nano_runProg(sample_read_dict, *args, method="qcat", inputflag="-f", outputf
         sample_read_dict ([type]): [description]
         method (str, optional): [description]. Defaults to "qcat".
         forceout: if the program do not create out file, you can set it to True
-        stdin: standard in format: load options first
+        stdin: standard in format: load options first, default false
         suffix_change: change file type
         outfmt: output format to change to
         exist_ok: if override existing folder
         switchinput_output: some program where stdin is the last command
+        xlabel: change name on the output file
         catchstdout: if to catch stdout in a file, stdout file name will be the same as output name, except for the suffix, you can change it with stdout_fmt
         stdout_fmt: what file type you want to store stdout
 
@@ -461,13 +570,14 @@ def changeFastqHeader(fastq, newrecordname=None, n=4, out="changeFastqHeader.fas
 
 
 
-def nano_modiheader(sample_read_dict, filetype="fasta", out="Modified_header", exist_ok=True):
+def nano_modiheader(sample_read_dict, filetype="fasta", out="Modified_header", forqiime2=False, exist_ok=True):
     """Modify read name to SampleID|BC|readid
 
     Args:
         sample_read_dict ([type]): [description]
         filetype (str, optional): [description]. Defaults to "fasta".
         out (str, optional): [description]. Defaults to "Modified_header".
+        forqiime2: modify headers as specified in qiime2 sample-id_seq-id
         exist_ok (bool, optional): [description]. Defaults to True.
     """
 
@@ -486,37 +596,71 @@ def nano_modiheader(sample_read_dict, filetype="fasta", out="Modified_header", e
     # each read would have an ID >sample_folder|barcode|readID
     # here barcode would be converted to barcode 1 to BC1
 
-    for sample, reads in sample_read_dict.items():
-        sample_folder = os.path.basename(sample)
+    if forqiime2:
+        for sample, reads in sample_read_dict.items():
+            sample_folder = os.path.basename(sample)
 
-        sample_folder_path = os.path.join(outpath, sample_folder)
+            sample_folder_path = os.path.join(outpath, sample_folder)
 
-        if os.path.exists(sample_folder_path):
-            print("Sample exist, will overide")
+            if os.path.exists(sample_folder_path):
+                print("Sample exist, will overide")
 
-        os.makedirs(sample_folder_path, exist_ok=True)
+            os.makedirs(sample_folder_path, exist_ok=True)
 
-        for read in reads:
-            barcode = os.path.basename(read)
-            
-            newbarcode = regexBarcode(barcode)
-            header1 = "|".join([sample_folder, newbarcode])
-            outputname = sample_folder + "_" + newbarcode + f".{filetype}"
-            barcode_path = os.path.join(sample_folder_path, outputname)
+            for read in reads:
+                barcode = os.path.basename(read)
+                
+                newbarcode = regexBarcode(barcode)
+                header1 = "_".join([sample_folder, newbarcode])
+                outputname = sample_folder + "_" + newbarcode + f".{filetype}"
+                barcode_path = os.path.join(sample_folder_path, outputname)
 
-            read_mapping[sample_folder_path].append(barcode_path)
+                read_mapping[sample_folder_path].append(barcode_path)
 
-            if filetype in ["fasta","fa", "fna"]:
-                seqRecord = SeqIO.parse(read, format= filetype) # this is a generator
-                with open(barcode_path, "w") as outf:
-                    for record in seqRecord:
-                        record.id = header1 + "|" + record.id
-                        seqs = record.seq 
-                        print(">"+record.id, sep="", file=outf)
-                        print(seqs, sep="", file=outf)
+                if filetype in ["fasta","fa", "fna"]:
+                    seqRecord = SeqIO.parse(read, format= filetype) # this is a generator
+                    with open(barcode_path, "w") as outf:
+                        for record in seqRecord:
+                            record.id = header1 + "_" + record.id
+                            seqs = record.seq 
+                            print(">"+record.id, sep="", file=outf)
+                            print(seqs, sep="", file=outf)
 
-            elif filetype in ["fastq", "fq"]:
-                changeFastqHeader(read, newrecordname=header1, out=barcode_path)
+                elif filetype in ["fastq", "fq"]:
+                    changeFastqHeader(read, newrecordname=header1, out=barcode_path)
+    else:
+
+        for sample, reads in sample_read_dict.items():
+            sample_folder = os.path.basename(sample)
+
+            sample_folder_path = os.path.join(outpath, sample_folder)
+
+            if os.path.exists(sample_folder_path):
+                print("Sample exist, will overide")
+
+            os.makedirs(sample_folder_path, exist_ok=True)
+
+            for read in reads:
+                barcode = os.path.basename(read)
+                
+                newbarcode = regexBarcode(barcode)
+                header1 = "|".join([sample_folder, newbarcode])
+                outputname = sample_folder + "_" + newbarcode + f".{filetype}"
+                barcode_path = os.path.join(sample_folder_path, outputname)
+
+                read_mapping[sample_folder_path].append(barcode_path)
+
+                if filetype in ["fasta","fa", "fna"]:
+                    seqRecord = SeqIO.parse(read, format= filetype) # this is a generator
+                    with open(barcode_path, "w") as outf:
+                        for record in seqRecord:
+                            record.id = header1 + "|" + record.id
+                            seqs = record.seq 
+                            print(">"+record.id, sep="", file=outf)
+                            print(seqs, sep="", file=outf)
+
+                elif filetype in ["fastq", "fq"]:
+                    changeFastqHeader(read, newrecordname=header1, out=barcode_path)
 
     return read_mapping
 
